package com.example.chatwithopenpgp

import android.os.Bundle
import android.os.Environment
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.EditText
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.lifecycle.lifecycleScope
import kotlinx.coroutines.CoroutineExceptionHandler
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import org.jivesoftware.smack.ConnectionConfiguration
import org.jivesoftware.smack.packet.Message
import org.jivesoftware.smack.tcp.XMPPTCPConnection
import org.jivesoftware.smack.tcp.XMPPTCPConnectionConfiguration
import org.jivesoftware.smackx.ox.OpenPgpManager
import org.jivesoftware.smackx.ox.OpenPgpSelf
import org.jivesoftware.smackx.ox.crypto.OpenPgpProvider
import org.jivesoftware.smackx.ox.crypto.PainlessOpenPgpProvider
import org.jivesoftware.smackx.ox.store.definition.OpenPgpStore
import org.jivesoftware.smackx.ox.store.filebased.FileBasedOpenPgpStore
import org.jivesoftware.smackx.ox_im.OXInstantMessagingManager
import org.jxmpp.jid.BareJid
import org.jxmpp.jid.impl.JidCreate
import org.pgpainless.key.OpenPgpV4Fingerprint
import java.io.File


class MainActivity : AppCompatActivity() {

    private var connection: XMPPTCPConnection? = null
    private var store: OpenPgpStore? = null
    private var provider: OpenPgpProvider? = null
    private var openPgpManager: OpenPgpManager? = null
    private var oxManager: OXInstantMessagingManager? = null
    private val self: OpenPgpSelf? = null
    private var username: String? = null

    private val handler by lazy {
        CoroutineExceptionHandler { _, throwable ->
            lifecycleScope.launch(Dispatchers.Main) {
                Toast.makeText(this@MainActivity, throwable.message, Toast.LENGTH_LONG).show()
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        val usernameText = findViewById<EditText>(R.id.username)
        val passwordText = findViewById<EditText>(R.id.password)

        val loginBtn = findViewById<Button>(R.id.login)
        val editorBtn = findViewById<Button>(R.id.sendBtn)

        val loginPanel = findViewById<ViewGroup>(R.id.linearLayout)
        val editorPanel = findViewById<ViewGroup>(R.id.editorLayout)

        loginBtn.setOnClickListener {
            lifecycleScope.launch(Dispatchers.IO + handler) {
                val config = XMPPTCPConnectionConfiguration.builder()
                    .setUsernameAndPassword(
                        usernameText.text.toString(),
                        passwordText.text.toString()
                    )
                    .setXmppDomain("localhost")
                    .setHost("192.168.1.101")
                    .setPort(5222)
                    .setSecurityMode(ConnectionConfiguration.SecurityMode.disabled)
                    .enableDefaultDebugger()
                    .build()
                connection = XMPPTCPConnection(config)
                connection?.connect()?.login()
                connection?.let {
                    if (it.isAuthenticated) {
                        launch(Dispatchers.Main) {
                            loginPanel.visibility = View.GONE
                            editorPanel.visibility = View.VISIBLE
                        }
                        this@MainActivity.username = usernameText.text.toString()
                        initPGP()
                    }
                }
            }
        }

        editorBtn.setOnClickListener {
            val texEditor = findViewById<EditText>(R.id.editor)
            val chatWith = findViewById<EditText>(R.id.chatWith)

            val jid: BareJid = JidCreate.bareFrom("${chatWith.text}@localhost")
            val message: String = texEditor.text.toString()
            oxManager?.sendOxMessage(
                openPgpManager?.getOpenPgpContact(jid.asEntityBareJidIfPossible()),
                message
            )

            texEditor.setText("")
        }
    }

    private fun initPGP() {
        store = FileBasedOpenPgpStore(File(filesDir, username))
        provider = PainlessOpenPgpProvider(store)
        openPgpManager = OpenPgpManager.getInstanceFor(connection)
        openPgpManager?.openPgpProvider = provider

        preparePGP()
        oxManager = OXInstantMessagingManager.getInstanceFor(connection)

        oxManager?.addOxMessageListener { openPgpContact, message, signcryptElement, openPgpMetadata ->
            val body: Message.Body =
                signcryptElement.getExtension(Message.Body.ELEMENT, Message.Body.NAMESPACE)

            val messageList = findViewById<TextView>(R.id.messageList)
            messageList.append("-------------\n${body.message}\n-------------\n")
        }
    }

    private fun preparePGP() {
        openPgpManager?.openPgpSelf?.let { self ->
            if (self.hasSecretKeyAvailable()) {
                val fp = OpenPgpV4Fingerprint(self.signingKeyRing).toString()
                Toast.makeText(this, "You already have key $fp available", Toast.LENGTH_LONG).show()
                openPgpManager?.announceSupportAndPublish()
                oxManager?.announceSupportForOxInstantMessaging()
            } else {
                openPgpManager?.generateAndImportKeyPair(self.jid)
                openPgpManager?.announceSupportAndPublish()
                oxManager?.announceSupportForOxInstantMessaging()
            }
        }
    }
}